from ._finance_core import (
    Bar,
    AverageTrueRange,
    ExponentialMovingAverage,
    Maximum,
    Minimum,
    MovingAverageConvergenceDivergence,
    RelativeStrengthIndex,
    SimpleMovingAverage,
    TrueRange,
)

__all__ = [
    "Bar",
    "AverageTrueRange",
    "ExponentialMovingAverage",
    "Maximum",
    "Minimum",
    "MovingAverageConvergenceDivergence",
    "RelativeStrengthIndex",
    "SimpleMovingAverage",
    "TrueRange"
]



import finance_core as fc
import pytest


def test_init():

    with pytest.raises(ValueError, match="Period cannot be 0."):
        fc.RelativeStrengthIndex(0)


def test_next():

    rsi = fc.RelativeStrengthIndex(3)

    assert rsi.next(1.0) == 100.0
    assert round(rsi.next(0.5), 4) == 66.6667
    assert round(rsi.next(2.0), 4) == 83.3333
    assert round(rsi.next(1.5), 4) == 60


def test_reset():

    rsi = fc.RelativeStrengthIndex(3)

    assert rsi.next(1.0) == 100.0
    assert round(rsi.next(0.5), 4) == 66.6667

    rsi.reset()
    assert rsi.next(1.0) == 100.0
    assert round(rsi.next(0.5), 4) == 66.6667



import finance_core as fc
import pytest


def test_init():

    with pytest.raises(ValueError, match="Period cannot be 0."):
        fc.AverageTrueRange(0)


def test_next():

    atr = fc.AverageTrueRange(3)

    bar1 = fc.Bar(open=7.0, high=10.0, low=7.0, close=9.0, volume=0)
    bar2 = fc.Bar(open=9.0, high=12.0, low=8.0, close=10.0, volume=0)
    bar3 = fc.Bar(open=10.0, high=15.0, low=9.0, close=11.0, volume=0)
    bar4 = fc.Bar(open=11.0, high=18.5, low=8.5, close=10.5, volume=0)

    assert atr.next(bar1) == 3.0
    assert atr.next(bar2) == 3.5
    assert atr.next(bar3) == 4.75
    assert atr.next(bar4) == 7.375


def test_reset():

    atr = fc.AverageTrueRange(3)

    bar1 = fc.Bar(open=7.0, high=10.0, low=7.0, close=9.0, volume=0)
    assert atr.next(bar1) == 3.0

    atr.reset()
    bar2 = fc.Bar(open=9.0, high=12.0, low=8.0, close=10.0, volume=0)
    assert atr.next(bar2) == 4.0

    atr.reset()
    bar3 = fc.Bar(open=10.0, high=15.0, low=9.0, close=11.0, volume=0)
    assert atr.next(bar3) == 6.0

    atr.reset()
    bar4 = fc.Bar(open=11.0, high=18.5, low=8.5, close=10.5, volume=0)
    assert atr.next(bar4) == 10.0



import finance_core as fc


def test_init():

    tr = fc.TrueRange()

    assert isinstance(tr, fc.TrueRange)


def test_next():

    tr = fc.TrueRange()

    bar1 = fc.Bar(
        open=1.0,
        high=3.0,
        low=1.0,
        close=3.0,
        volume=0
    )

    assert tr.next(bar1) == 2.0

    bar2 = fc.Bar(
        open=2.0,
        high=6.0,
        low=2.0,
        close=6.0,
        volume=0
    )

    assert tr.next(bar2) == 4.0


def test_reset():

    tr = fc.TrueRange()

    bar1 = fc.Bar(
        open=1.0,
        high=3.0,
        low=1.0,
        close=3.0,
        volume=0
    )

    assert tr.next(bar1) == 2.0

    tr.reset()
    bar2 = fc.Bar(
        open=2.0,
        high=1.0,
        low=2.0,
        close=1.0,
        volume=0
    )

    assert tr.next(bar2) == -1.0



import finance_core as fc
import pytest


def test_init():

    with pytest.raises(ValueError, match="Period cannot be 0."):
        fc.ExponentialMovingAverage(0)


def test_next():

    ema = fc.ExponentialMovingAverage(3)

    assert ema.next(2.0) == 2.0
    assert ema.next(5.0) == 3.5
    assert ema.next(7.0) == 5.25
    assert ema.next(6.0) == 5.625


def test_reset():

    ema = fc.ExponentialMovingAverage(3)

    assert ema.next(2.0) == 2.0
    assert ema.next(5.0) == 3.5
    assert ema.next(7.0) == 5.25
    assert ema.next(6.0) == 5.625

    ema.reset()
    assert ema.next(2.0) == 2.0

    ema.reset()
    assert ema.next(4.0) == 4.0



__all__ = [
    "Bar",
    "AverageTrueRange",
    "ExponentialMovingAverage",
    "Maximum",
    "Minimum",
    "MovingAverageConvergenceDivergence",
    "RelativeStrengthIndex",
    "SimpleMovingAverage",
    "TrueRange"
]


class Bar:
    def __init__(
            self,
            open: float,
            high: float,
            low: float,
            close: float,
            volume: int
    ) -> None:
        """Bar data item."""


class AverageTrueRange:
    def __init__(self, period: int) -> None:
        """Average true range."""

    def period(self) -> int:
        """Return the period of the smoothing."""

    def next(self, input: Bar) -> float:
        """Calculate average true range for the next period."""

    def reset(self) -> None:
        """Reset the current calculations."""


class ExponentialMovingAverage:
    def __init__(self, period: int) -> None:
        """Exponential moving average."""

    def period(self) -> int:
        """Return the number of periods."""

    def next(self, input: float) -> float:
        """Calculate the exponential moving average of the current periods."""

    def reset(self) -> None:
        """Reset the current calculations."""


class Maximum:
    def __init__(self, period: int) -> None:
        """Create a new maximum indicator."""

    def period(self) -> int:
        """Return the number of periods."""

    def next(self, input: float) -> float:
        """Calculate the maximum of the current periods."""

    def reset(self) -> None:
        """Reset the current calculations."""


class Minimum:
    def __init__(self, period: int) -> None:
        """Create a minimum indicator."""

    def period(self) -> int:
        """Return the number of periods."""

    def next(self, input: float) -> float:
        """Calculate the minimm of the current periods."""

    def reset(self) -> None:
        """Reset the current calculations."""


class MovingAverageConvergenceDivergence:
    def __init__(self, long_period: int, short_period: int, signal_period: int) -> None:
        """Moving average convergence divergence."""

    def next(self, input: float) -> tuple[float, float, float]:
        """Calculate the moving average convergence divergence of the current periods.

        Returns the MACD (0), signal (1), and histogram (2).
        """

    def reset(self) -> None:
        """Reset the current calculations."""


class RelativeStrengthIndex:
    def __init__(self, period: int) -> None:
        """Relative strength index."""

    def period(self) -> int:
        """Return the number of periods."""

    def next(self, input: float) -> float:
        """Calculate the relative strength index of the current periods."""

    def reset(self) -> None:
        """Reset the calculations."""


class SimpleMovingAverage:
    def __init__(self, period: int) -> None:
        """Create a simple moving average indicator."""

    def period(self) -> int:
        """Return the number of periods."""

    def next(self, input: float) -> float:
        """Calculate the simple moving average of the current periods."""

    def reset(self) -> None:
        """Reset the current calculations."""


class TrueRange:
    def __init__(self) -> None:
        """Create a true range indicator."""

    def next(self, input: Bar) -> float:
        """Calculate the true range."""

    def reset(self) -> None:
        """Reset the current true range."""



import finance_core as fc
import pytest


def test_init():

    with pytest.raises(ValueError, match="Period cannot be 0."):
        fc.SimpleMovingAverage(0)


def test_next():

    sma = fc.SimpleMovingAverage(3)

    assert sma.next(1.0) == 1.0
    assert sma.next(2.0) == 1.5
    assert sma.next(3.0) == 2.0
    assert sma.next(4.0) == 3.0


def test_reset():

    sma = fc.SimpleMovingAverage(3)

    assert sma.next(1.0) == 1.0

    sma.reset()
    assert sma.next(2.0) == 2.0

    sma.reset()
    assert sma.next(3.0) == 3.0



import finance_core as fc
import pytest


def _round(nums: tuple[float, float, float]) -> tuple[float, float, float]:
    """Round to four digits."""
    n0 = round(nums[0], 4)
    n1 = round(nums[1], 4)
    n2 = round(nums[2], 4)
    return (n0, n1, n2)


def test_init():

    with pytest.raises(ValueError, match="Period cannot be 0."):
        fc.MovingAverageConvergenceDivergence(0, 12, 9)

    with pytest.raises(ValueError, match="Period cannot be 0."):
        fc.MovingAverageConvergenceDivergence(26, 0, 9)

    with pytest.raises(ValueError, match="Period cannot be 0."):
        fc.MovingAverageConvergenceDivergence(26, 12, 0)


def test_next():

    macd = fc.MovingAverageConvergenceDivergence(26, 12, 9)

    assert macd.next(3.0) == (0.0, 0.0, 0.0)
    assert _round(macd.next(4.0)) == (0.0798, 0.016, 0.0638)
    assert _round(macd.next(5.0)) == (0.2211, 0.0570, 0.1641)
    assert _round(macd.next(6.0)) == (0.4091, 0.1274, 0.2817)


def test_reset():

    macd = fc.MovingAverageConvergenceDivergence(26, 12, 9)

    assert _round(macd.next(3.0)) == (0.0, 0.0, 0.0)
    assert _round(macd.next(4.0)) == (0.0798, 0.016, 0.0638)

    macd.reset()
    assert _round(macd.next(3.0)) == (0.0, 0.0, 0.0)
    assert _round(macd.next(4.0)) == (0.0798, 0.016, 0.0638)

    macd.reset()
    assert _round(macd.next(3.0)) == (0.0, 0.0, 0.0)
    assert _round(macd.next(4.0)) == (0.0798, 0.016, 0.0638)



import finance_core as fc


max = fc.Maximum(3)

print(max.next(1))
print(max.next(4))
print(max.next(2))
print(max.next(5))


min = fc.Minimum(3)

print(min.next(1))
print(min.next(4))
print(min.next(2))
print(min.next(5))


sma = fc.SimpleMovingAverage(3)

print(sma.next(1))
print(sma.next(4))
print(sma.next(2))
print(sma.next(5))

macd = fc.MovingAverageConvergenceDivergence(26, 12, 9)

print(macd.next(1))
print(macd.next(4))
print(macd.next(2))
print(macd.next(5))



import finance_core as fc
import pytest


def test_init():

    with pytest.raises(ValueError, match="Period cannot be 0."):
        fc.Minimum(0)


def test_next():

    min = fc.Minimum(3)

    assert min.next(4.0) == 4.0
    assert min.next(3.0) == 3.0
    assert min.next(2.0) == 2.0
    assert min.next(5.0) == 2.0


def test_reset():

    min = fc.Minimum(3)

    assert min.next(3.0) == 3.0

    min.reset()
    assert min.next(4.0) == 4.0

    min.reset()
    assert min.next(5.0) == 5.0



import finance_core as fc
import pytest


def test_init():

    with pytest.raises(ValueError, match="Period cannot be 0."):
        fc.Maximum(0)


def test_next():

    max = fc.Maximum(3)

    assert max.next(5.0) == 5.0
    assert max.next(2.0) == 5.0
    assert max.next(3.0) == 5.0
    assert max.next(4.0) == 4.0


def test_reset():

    max = fc.Maximum(3)

    assert max.next(5.0) == 5.0

    max.reset()
    assert max.next(4.0) == 4.0

    max.reset()
    assert max.next(3.0) == 3.0
